@@ -1,17 +1,21 @@
 import logging
 from collections.abc import Mapping
 from copy import copy
+from typing import Any, Dict, List, Optional, Set, Union, Literal, Type, TYPE_CHECKING
 from deepdiff.helper import (
-    RemapDict, strings, short_repr, notpresent, get_type, numpy_numbers, np, literal_eval_extended,
+    RemapDict, strings, notpresent, get_type, numpy_numbers, np, literal_eval_extended,
     dict_, SetOrdered)
 from deepdiff.path import stringify_element
 
+if TYPE_CHECKING:
+    from deepdiff.diff import DeepDiff
+
 logger = logging.getLogger(__name__)
 
-FORCE_DEFAULT = 'fake'
-UP_DOWN = {'up': 'down', 'down': 'up'}
+FORCE_DEFAULT: Literal['fake'] = 'fake'
+UP_DOWN: Dict[str, str] = {'up': 'down', 'down': 'up'}
 
-REPORT_KEYS = {
+REPORT_KEYS: Set[str] = {
     "type_changes",
     "dictionary_item_added",
     "dictionary_item_removed",
@@ -27,7 +31,7 @@ REPORT_KEYS = {
     "repetition_change",
 }
 
-CUSTOM_FIELD = "__internal:custom:extra_info"
+CUSTOM_FIELD: str = "__internal:custom:extra_info"
 
 
 class DoesNotExist(Exception):
@@ -36,7 +40,7 @@ class DoesNotExist(Exception):
 
 class ResultDict(RemapDict):
 
-    def remove_empty_keys(self):
+    def remove_empty_keys(self) -> None:
         """
         Remove empty keys from this object. Should always be called after the result is final.
         :return:
@@ -48,11 +52,11 @@ class ResultDict(RemapDict):
 
 
 class TreeResult(ResultDict):
-    def __init__(self):
+    def __init__(self) -> None:
         for key in REPORT_KEYS:
             self[key] = SetOrdered()
 
-    def mutual_add_removes_to_become_value_changes(self):
+    def mutual_add_removes_to_become_value_changes(self) -> None:
         """
         There might be the same paths reported in the results as removed and added.
         In such cases they should be reported as value_changes.
@@ -62,32 +66,38 @@ class TreeResult(ResultDict):
 
         This function should only be run on the Tree Result.
         """
-        if self.get('iterable_item_added') and self.get('iterable_item_removed'):
-            added_paths = {i.path(): i for i in self['iterable_item_added']}
-            removed_paths = {i.path(): i for i in self['iterable_item_removed']}
+        iterable_item_added = self.get('iterable_item_added')
+        iterable_item_removed = self.get('iterable_item_removed')
+        if iterable_item_added is not None and iterable_item_removed is not None:
+            added_paths = {i.path(): i for i in iterable_item_added}
+            removed_paths = {i.path(): i for i in iterable_item_removed}
             mutual_paths = set(added_paths) & set(removed_paths)
 
-            if mutual_paths and 'values_changed' not in self:
+            if mutual_paths and 'values_changed' not in self or self['values_changed'] is None:
                 self['values_changed'] = SetOrdered()
             for path in mutual_paths:
                 level_before = removed_paths[path]
-                self['iterable_item_removed'].remove(level_before)
+                iterable_item_removed.remove(level_before)
                 level_after = added_paths[path]
-                self['iterable_item_added'].remove(level_after)
+                iterable_item_added.remove(level_after)
                 level_before.t2 = level_after.t2
-                self['values_changed'].add(level_before)
+                self['values_changed'].add(level_before)  # type: ignore
                 level_before.report_type = 'values_changed'
-        if 'iterable_item_removed' in self and not self['iterable_item_removed']:
+        if 'iterable_item_removed' in self and not iterable_item_removed:
             del self['iterable_item_removed']
-        if 'iterable_item_added' in self and not self['iterable_item_added']:
+        if 'iterable_item_added' in self and not iterable_item_added:
             del self['iterable_item_added']
 
-    def __getitem__(self, item):
+    def __getitem__(self, item: str) -> SetOrdered:
         if item not in self:
             self[item] = SetOrdered()
-        return self.get(item)
+        result = self.get(item)
+        if result is None:
+            result = SetOrdered()
+            self[item] = result
+        return result
 
-    def __len__(self):
+    def __len__(self) -> int:
         length = 0
         for value in self.values():
             if isinstance(value, SetOrdered):
@@ -98,9 +108,9 @@ class TreeResult(ResultDict):
 
 
 class TextResult(ResultDict):
-    ADD_QUOTES_TO_STRINGS = True
+    ADD_QUOTES_TO_STRINGS: bool = True
 
-    def __init__(self, tree_results=None, verbose_level=1):
+    def __init__(self, tree_results: Optional['TreeResult'] = None, verbose_level: int = 1) -> None:
         self.verbose_level = verbose_level
         # TODO: centralize keys
         self.update({
@@ -122,10 +132,10 @@ class TextResult(ResultDict):
         if tree_results:
             self._from_tree_results(tree_results)
 
-    def __set_or_dict(self):
+    def __set_or_dict(self) -> Union[Dict[str, Any], SetOrdered]:
         return {} if self.verbose_level >= 2 else SetOrdered()
 
-    def _from_tree_results(self, tree):
+    def _from_tree_results(self, tree: 'TreeResult') -> None:
         """
         Populate this object by parsing an existing reference-style result dictionary.
         :param tree: A TreeResult
@@ -147,7 +157,7 @@ class TextResult(ResultDict):
         self._from_tree_deep_distance(tree)
         self._from_tree_custom_results(tree)
 
-    def _from_tree_default(self, tree, report_type, ignore_if_in_iterable_opcodes=False):
+    def _from_tree_default(self, tree: 'TreeResult', report_type: str, ignore_if_in_iterable_opcodes: bool = False) -> None:
         if report_type in tree:
                 
             for change in tree[report_type]:  # report each change
@@ -219,10 +229,11 @@ class TextResult(ResultDict):
 
     def _from_tree_iterable_item_moved(self, tree):
         if 'iterable_item_moved' in tree and self.verbose_level > 1:
+
             for change in tree['iterable_item_moved']:
-                the_changed = {'new_path': change.path(use_t2=True), 'value': change.t2}
+                the_changed = {'new_path': change.path(use_t2=True, reporting_move=True), 'value': change.t2}
                 self['iterable_item_moved'][change.path(
-                    force=FORCE_DEFAULT)] = the_changed
+                    force=FORCE_DEFAULT, use_t2=False, reporting_move=True)] = the_changed
 
     def _from_tree_unprocessed(self, tree):
         if 'unprocessed' in tree:
@@ -242,7 +253,7 @@ class TextResult(ResultDict):
                     item = "'%s'" % item
                 if is_dict:
                     if path not in set_item_info:
-                        set_item_info[path] = set()
+                        set_item_info[path] = set()  # type: ignore
                     set_item_info[path].add(item)
                 else:
                     set_item_info.add("{}[{}]".format(path, str(item)))
@@ -288,9 +299,9 @@ class TextResult(ResultDict):
 
 
 class DeltaResult(TextResult):
-    ADD_QUOTES_TO_STRINGS = False
+    ADD_QUOTES_TO_STRINGS: bool = False
 
-    def __init__(self, tree_results=None, ignore_order=None, always_include_values=False, _iterable_opcodes=None):
+    def __init__(self, tree_results: Optional['TreeResult'] = None, ignore_order: Optional[bool] = None, always_include_values: bool = False, _iterable_opcodes: Optional[Dict[str, Any]] = None) -> None:
         self.ignore_order = ignore_order
         self.always_include_values = always_include_values
 
@@ -426,11 +437,11 @@ class DeltaResult(TextResult):
         if 'iterable_item_moved' in tree:
             for change in tree['iterable_item_moved']:
                 if (
-                    change.up.path(force=FORCE_DEFAULT) not in self["_iterable_opcodes"]
+                    change.up.path(force=FORCE_DEFAULT, reporting_move=True) not in self["_iterable_opcodes"]
                 ):
-                    the_changed = {'new_path': change.path(use_t2=True), 'value': change.t2}
+                    the_changed = {'new_path': change.path(use_t2=True, reporting_move=True), 'value': change.t2}
                     self['iterable_item_moved'][change.path(
-                        force=FORCE_DEFAULT)] = the_changed
+                        force=FORCE_DEFAULT, reporting_move=True)] = the_changed
 
 
 class DiffLevel:
@@ -514,15 +525,15 @@ class DiffLevel:
     """
 
     def __init__(self,
-                 t1,
-                 t2,
-                 down=None,
-                 up=None,
-                 report_type=None,
-                 child_rel1=None,
-                 child_rel2=None,
-                 additional=None,
-                 verbose_level=1):
+                 t1: Any,
+                 t2: Any,
+                 down: Optional['DiffLevel'] = None,
+                 up: Optional['DiffLevel'] = None,
+                 report_type: Optional[str] = None,
+                 child_rel1: Optional['ChildRelationship'] = None,
+                 child_rel2: Optional['ChildRelationship'] = None,
+                 additional: Optional[Dict[str, Any]] = None,
+                 verbose_level: int = 1) -> None:
         """
         :param child_rel1: Either:
                             - An existing ChildRelationship object describing the "down" relationship for t1; or
@@ -578,20 +589,22 @@ class DiffLevel:
 
         self.verbose_level = verbose_level
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         if self.verbose_level:
+            from deepdiff.summarize import summarize
+
             if self.additional:
-                additional_repr = short_repr(self.additional, max_length=35)
+                additional_repr = summarize(self.additional, max_length=35)
                 result = "<{} {}>".format(self.path(), additional_repr)
             else:
-                t1_repr = short_repr(self.t1)
-                t2_repr = short_repr(self.t2)
+                t1_repr = summarize(self.t1, max_length=35)
+                t2_repr = summarize(self.t2, max_length=35)
                 result = "<{} t1:{}, t2:{}>".format(self.path(), t1_repr, t2_repr)
         else:
             result = "<{}>".format(self.path())
         return result
 
-    def __setattr__(self, key, value):
+    def __setattr__(self, key: str, value: Any) -> None:
         # Setting up or down, will set the opposite link in this linked list.
         if key in UP_DOWN and value is not None:
             self.__dict__[key] = value
@@ -600,15 +613,15 @@ class DiffLevel:
         else:
             self.__dict__[key] = value
 
-    def __iter__(self):
+    def __iter__(self) -> Any:
         yield self.t1
         yield self.t2
 
     @property
-    def repetition(self):
+    def repetition(self) -> Dict[str, Any]:
         return self.additional['repetition']
 
-    def auto_generate_child_rel(self, klass, param, param2=None):
+    def auto_generate_child_rel(self, klass: Type['ChildRelationship'], param: Any, param2: Optional[Any] = None) -> None:
         """
         Auto-populate self.child_rel1 and self.child_rel2.
         This requires self.down to be another valid DiffLevel object.
@@ -617,15 +630,15 @@ class DiffLevel:
         :param param: A ChildRelationship subclass-dependent parameter describing how to get from parent to child,
                       e.g. the key in a dict
         """
-        if self.down.t1 is not notpresent:
+        if self.down.t1 is not notpresent:  # type: ignore
             self.t1_child_rel = ChildRelationship.create(
-                klass=klass, parent=self.t1, child=self.down.t1, param=param)
-        if self.down.t2 is not notpresent:
+                klass=klass, parent=self.t1, child=self.down.t1, param=param)  # type: ignore
+        if self.down.t2 is not notpresent:  # type: ignore
             self.t2_child_rel = ChildRelationship.create(
-                klass=klass, parent=self.t2, child=self.down.t2, param=param if param2 is None else param2)
+                klass=klass, parent=self.t2, child=self.down.t2, param=param if param2 is None else param2)  # type: ignore
 
     @property
-    def all_up(self):
+    def all_up(self) -> 'DiffLevel':
         """
         Get the root object of this comparison.
         (This is a convenient wrapper for following the up attribute as often as you can.)
@@ -637,7 +650,7 @@ class DiffLevel:
         return level
 
     @property
-    def all_down(self):
+    def all_down(self) -> 'DiffLevel':
         """
         Get the leaf object of this comparison.
         (This is a convenient wrapper for following the down attribute as often as you can.)
@@ -649,10 +662,10 @@ class DiffLevel:
         return level
 
     @staticmethod
-    def _format_result(root, result):
+    def _format_result(root: str, result: Optional[str]) -> Optional[str]:
         return None if result is None else "{}{}".format(root, result)
 
-    def get_root_key(self, use_t2=False):
+    def get_root_key(self, use_t2: bool = False) -> Any:
         """
         Get the path's root key value for this change
 
@@ -669,7 +682,7 @@ class DiffLevel:
             return next_rel.param
         return notpresent
 
-    def path(self, root="root", force=None, get_parent_too=False, use_t2=False, output_format='str'):
+    def path(self, root: str = "root", force: Optional[str] = None, get_parent_too: bool = False, use_t2: bool = False, output_format: Literal['str', 'list'] = 'str', reporting_move: bool = False) -> Any:
         """
         A python syntax string describing how to descend to this level, assuming the top level object is called root.
         Returns None if the path is not representable as a string.
@@ -695,6 +708,9 @@ class DiffLevel:
         :param output_format: The format of the output. The options are 'str' which is the default and produces a
                               string representation of the path or 'list' to produce a list of keys and attributes
                               that produce the path.
+
+        :param reporting_move: This should be set to true if and only if we are reporting on iterable_item_moved.
+                               All other cases should leave this set to False.
         """
         # TODO: We could optimize this by building on top of self.up's path if it is cached there
         cache_key = "{}{}{}{}".format(force, get_parent_too, use_t2, output_format)
@@ -716,7 +732,16 @@ class DiffLevel:
         # traverse all levels of this relationship
         while level and level is not self:
             # get this level's relationship object
-            if use_t2:
+            if level.additional.get("moved") and not reporting_move:
+                # To ensure we can properly replay items such as values_changed in items that may have moved, we
+                # need to make sure that all paths are reported relative to t2 if a level has reported a move.
+                # If we are reporting a move, the path is already correct and does not need to be swapped.
+                # Additional context of "moved" is only ever set if using iterable_compare_func and a move has taken place.
+                level_use_t2 = not use_t2
+            else:
+                level_use_t2 = use_t2
+
+            if level_use_t2:
                 next_rel = level.t2_child_rel or level.t1_child_rel
             else:
                 next_rel = level.t1_child_rel or level.t2_child_rel  # next relationship object to get a formatted param from
@@ -737,29 +762,29 @@ class DiffLevel:
                     result = None
                     break
             elif output_format == 'list':
-                result.append(next_rel.param)
+                result.append(next_rel.param)  # type: ignore
 
             # Prepare processing next level
             level = level.down
 
         if output_format == 'str':
             if get_parent_too:
-                self._path[cache_key] = (parent, param, result)
-                output = (self._format_result(root, parent), param, self._format_result(root, result))
+                self._path[cache_key] = (parent, param, result)  # type: ignore
+                output = (self._format_result(root, parent), param, self._format_result(root, result))  # type: ignore
             else:
                 self._path[cache_key] = result
-                output = self._format_result(root, result)
+                output = self._format_result(root, result) if isinstance(result, (str, type(None))) else None
         else:
             output = result
         return output
 
     def create_deeper(self,
-                      new_t1,
-                      new_t2,
-                      child_relationship_class,
-                      child_relationship_param=None,
-                      child_relationship_param2=None,
-                      report_type=None):
+                      new_t1: Any,
+                      new_t2: Any,
+                      child_relationship_class: Type['ChildRelationship'],
+                      child_relationship_param: Optional[Any] = None,
+                      child_relationship_param2: Optional[Any] = None,
+                      report_type: Optional[str] = None) -> 'DiffLevel':
         """
         Start a new comparison level and correctly link it to this one.
         :rtype: DiffLevel
@@ -774,12 +799,12 @@ class DiffLevel:
         return result
 
     def branch_deeper(self,
-                      new_t1,
-                      new_t2,
-                      child_relationship_class,
-                      child_relationship_param=None,
-                      child_relationship_param2=None,
-                      report_type=None):
+                      new_t1: Any,
+                      new_t2: Any,
+                      child_relationship_class: Type['ChildRelationship'],
+                      child_relationship_param: Optional[Any] = None,
+                      child_relationship_param2: Optional[Any] = None,
+                      report_type: Optional[str] = None) -> 'DiffLevel':
         """
         Branch this comparison: Do not touch this comparison line, but create a new one with exactly the same content,
         just one level deeper.
@@ -790,7 +815,7 @@ class DiffLevel:
         return branch.create_deeper(new_t1, new_t2, child_relationship_class,
                                     child_relationship_param, child_relationship_param2, report_type)
 
-    def copy(self):
+    def copy(self) -> 'DiffLevel':
         """
         Get a deep copy of this comparision line.
         :return: The leaf ("downmost") object of the copy.
@@ -833,20 +858,20 @@ class ChildRelationship:
 
     # Format to a be used for representing param.
     # E.g. for a dict, this turns a formatted param param "42" into "[42]".
-    param_repr_format = None
+    param_repr_format: Optional[str] = None
 
     # This is a hook allowing subclasses to manipulate param strings.
     # :param string: Input string
     # :return: Manipulated string, as appropriate in this context.
-    quote_str = None
+    quote_str: Optional[str] = None
 
     @staticmethod
-    def create(klass, parent, child, param=None):
+    def create(klass: Type['ChildRelationship'], parent: Any, child: Any, param: Optional[Any] = None) -> 'ChildRelationship':
         if not issubclass(klass, ChildRelationship):
             raise TypeError
         return klass(parent, child, param)
 
-    def __init__(self, parent, child, param=None):
+    def __init__(self, parent: Any, child: Any, param: Optional[Any] = None) -> None:
         # The parent object of this relationship, e.g. a dict
         self.parent = parent
 
@@ -856,14 +881,16 @@ class ChildRelationship:
         # A subclass-dependent parameter describing how to get from parent to child, e.g. the key in a dict
         self.param = param
 
-    def __repr__(self):
+    def __repr__(self) -> str:
+        from deepdiff.summarize import summarize
+
         name = "<{} parent:{}, child:{}, param:{}>"
-        parent = short_repr(self.parent)
-        child = short_repr(self.child)
-        param = short_repr(self.param)
+        parent = summarize(self.parent, max_length=35)
+        child = summarize(self.child, max_length=35)
+        param = summarize(self.param, max_length=15)
         return name.format(self.__class__.__name__, parent, child, param)
 
-    def get_param_repr(self, force=None):
+    def get_param_repr(self, force: Optional[str] = None) -> Optional[str]:
         """
         Returns a formatted param python parsable string describing this relationship,
         or None if the relationship is not representable as a string.
@@ -880,7 +907,7 @@ class ChildRelationship:
         """
         return self.stringify_param(force)
 
-    def stringify_param(self, force=None):
+    def stringify_param(self, force: Optional[str] = None) -> Optional[str]:
         """
         Convert param to a string. Return None if there is no string representation.
         This is called by get_param_repr()
@@ -903,7 +930,7 @@ class ChildRelationship:
         elif isinstance(param, tuple):  # Currently only for numpy ndarrays
             result = ']['.join(map(repr, param))
         elif hasattr(param, '__dataclass_fields__'):
-            attrs_to_values = [f"{key}={value}" for key, value in [(i, getattr(param, i)) for i in param.__dataclass_fields__]]
+            attrs_to_values = [f"{key}={value}" for key, value in [(i, getattr(param, i)) for i in param.__dataclass_fields__]]  # type: ignore
             result = f"{param.__class__.__name__}({','.join(attrs_to_values)})"
         else:
             candidate = repr(param)
@@ -927,13 +954,13 @@ class ChildRelationship:
 
 
 class DictRelationship(ChildRelationship):
-    param_repr_format = "[{}]"
-    quote_str = "'{}'"
+    param_repr_format: Optional[str] = "[{}]"
+    quote_str: Optional[str] = "'{}'"
 
 
 class NumpyArrayRelationship(ChildRelationship):
-    param_repr_format = "[{}]"
-    quote_str = None
+    param_repr_format: Optional[str] = "[{}]"
+    quote_str: Optional[str] = None
 
 
 class SubscriptableIterableRelationship(DictRelationship):
@@ -951,9 +978,9 @@ class SetRelationship(InaccessibleRelationship):
 
 class NonSubscriptableIterableRelationship(InaccessibleRelationship):
 
-    param_repr_format = "[{}]"
+    param_repr_format: Optional[str] = "[{}]"
 
-    def get_param_repr(self, force=None):
+    def get_param_repr(self, force: Optional[str] = None) -> Optional[str]:
         if force == 'yes':
             result = "(unrepresentable)"
         elif force == 'fake' and self.param:
@@ -965,4 +992,4 @@ class NonSubscriptableIterableRelationship(InaccessibleRelationship):
 
 
 class AttributeRelationship(ChildRelationship):
-    param_repr_format = ".{}"
+    param_repr_format: Optional[str] = ".{}"
